/* Unit testing for outcomes
(C) 2013-2024 Niall Douglas <http://www.nedproductions.biz/> (5 commits)


Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#include <boost/outcome.hpp>
#include <boost/outcome/try.hpp>
#include <boost/test/unit_test.hpp>
#include <boost/test/unit_test_monitor.hpp>

BOOST_OUTCOME_AUTO_TEST_CASE(issues_0007_test, "BOOST_OUTCOME_TRYV(expr) in a function whose return outcome's type has no default constructor fails to compile")
{
  using namespace BOOST_OUTCOME_V2_NAMESPACE;
  struct udt  // NOLINT
  {
    explicit udt(int /*unused*/) {}
    //    udt() = delete;
    udt(const udt &) = default;
    udt(udt &&) = default;
  };
  {
    auto f = []() -> result<udt> {
      auto g = [] { return result<int>(5); };
      /* This fails because BOOST_OUTCOME_TRYV() returns a result<void>
      which if it were valued void, would implicitly convert into a
      default constructed udt which is not possible, hence the compile error.
      */
      BOOST_OUTCOME_TRYV(g());
      return udt(5);
    };
    (void) f();
  }
  {
    auto f = []() -> outcome<udt> {
      auto g = [] { return outcome<int>(5); };
      /* This fails because BOOST_OUTCOME_TRYV() returns a result<void>
      which if it were valued void, would implicitly convert into a
      default constructed udt which is not possible, hence the compile error.
      */
      BOOST_OUTCOME_TRYV(g());
      return udt(5);
    };
    (void) f();
  }
  {
    auto f = []() -> outcome<udt> {
      auto g = [] { return result<int>(5); };
      /* This fails because BOOST_OUTCOME_TRYV() returns a result<void>
      which if it were valued void, would implicitly convert into a
      default constructed udt which is not possible, hence the compile error.
      */
      BOOST_OUTCOME_TRYV(g());
      return udt(5);
    };
    (void) f();
  }
}
